;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Entity Component Include
;;       2B Position             {x, y}
;;       2B Velocity             {vx, vy}
;;       2B Size                 {h, w}
;;       2B Sprite Pointer       {sprite}
;;       2B Animation Pointer    {anim_ptr}
;;       1B Animation Status     {anim_st}
;;       2B Enemy's Route        {route_init, route_fin}
;;       2B Last Video Pointer   {lastVP}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Entity DEFINITION MACRO
  .macro DefineCmp_Entity  _type, _collider, _x, _y, _vx, _vy, _h, _w, _sprite, _anim_ptr, _route_init, _route_fin   
      .db _type            ;; Component type
      .db _collider        ;; Collider type
      .db _x, _y           ;; Positions
      .db _vx, _vy         ;; Velocities
      .db _h, _w           ;; Size
      .dw _sprite          ;; Sprite Pointer
      .dw _anim_ptr        ;; Animation Pointer
      .db 0x00             ;; Animation Status
      .db _route_init      ;; Enemy's route init
      .db _route_fin       ;; Enemy's route fin
      .dw 0xCCCC           ;; Last Video Memory Pointer Value
  .endm

 ;; Entity Struct (Field offsets)
  e_type                = 0      ;; 00 {1} - Entity Type 
  e_collider            = 1      ;; 01 {1} - Collider type
  e_x               		= 2      ;; 02 {1} - Entity X Screen Coordinate
  e_y               		= 3      ;; 03 {1} - Entity Y Screen Coordinate
  e_vx              		= 4      ;; 04 {1} - Entity X Velocity
  e_vy              		= 5      ;; 05 {1} - Entity Y Velocity
  e_h               		= 6      ;; 06 {1} - Entity Height
  e_w               		= 7      ;; 07 {1} - Entity Width (Bytes)
  e_sprite_l        		= 8      ;; 08 {1} - Entity Sprite Pointer
  e_sprite_h        		= 9      ;; 09 {1}
  e_anim_ptr_l      		= 10     ;; 10 {1} - Animation Pointer
  e_anim_ptr_h      		= 11     ;; 11 {1}
  e_anim_st             = 12     ;; 12 {1} - Animation Status
  e_route_init          = 13     ;; 13 {1} - Enemy's route
  e_route_fin           = 14     ;; 14 {1}
  e_lastVP_l        		= 15     ;; 15 {1} - Entity Previous VideoMem
  e_lastVP_h        		= 16     ;; 16 {1} 
  sizeof_e              = 17     ;; sizeof (Entity Struct)
  neg_sizeof_e          = -17    ;; - sizeof (Entity Struct) use for destroy

;;  DURL
 ;; 0000 inactive
 ;; 0001 left
 ;; 0010 right
 ;; 0011 --
 ;; 0100 jump
 ;; 0101 jump-left
 ;; 0110 jump-right
 ;; 0111 --
 ;; 1000 down
 ;; 1001 down-left
 ;; 1010 down-right

;; Animation Status
  e_anim_st_inactive    = 0
  e_anim_st_left        = 1
  e_anim_st_right       = 2
  e_anim_st_jump        = 3
  e_anim_st_go_down     = 4

 ;; Collider Types
  e_collider_none   = 0
  e_collider_enemy  = 1
  e_collider_door   = 2
  e_collider_candy  = 3

;; Mask bit numbers
  e_type_physics_bit    = 6
  e_type_render_bit     = 5
  e_type_animation_bit  = 4
  e_type_collider_bit   = 3
  e_type_ai_bit         = 2

 ;; Entity Types (masks)
  e_type_invalid        = 0x00                 
  e_type_physics_mask   = (1 << e_type_physics_bit)   ;; 0x20
  e_type_render_mask    = (1 << e_type_render_bit)    ;; 0x10
  e_type_animation_mask = (1 << e_type_animation_bit) ;; 0x08
  e_type_collider_mask  = (1 << e_type_collider_bit)  ;; 0x04
  e_type_ai_mask        = (1 << e_type_ai_bit)        ;; 0x02
  e_type_dead           = 0x80  ;; upper bit signals dead entity
  e_type_default        = 0x7F  ;; all bits = 1, exept the upper one

 ;; Utility definitions
  nullptr = #0x0000
