;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_array.h.s"
.include "cmp/macros_entity.h.s"

.include "man/entity.h.s"

.include "assets/assets.h.s"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Entity_Manager
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CONSTANTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SIZE_ALL_ENTITIES = sizeof_e * MAX_ENTITIES

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VALUES IN MEMORY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
DefineComponentArrayStructure _entity, MAX_ENTITIES, sizeof_e

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_GETARRAY
;; DESTROYS: A, IX
;; RETURN:
;;       A: The array's total number of newly formed entities
;;      IX: Pointer to the start of the array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_getArray::
   ld    ix, #_entity_array
   ld    a, (#_entity_num)
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_GETENTITYIX
;; INPUT:
;;    C: Collider type
;; DESTROYS: A, IX, C
;; RETURN:
;;    IX: Pointer to the entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_getEntityIX::
   ld    c, a
   call  man_entity_getArray
loop:
   ;;----------------------------------------------------
   ;; Check the end of the array and if it is, 
   ;; then returns
   ;;
   ld    a, (ix)        ;; a = entity type
   or    a
   ret   z

   ld    b, e_collider(ix)
   ld    a, c
   cp    b
   ret   z

   ld    de, #sizeof_e
   add   ix, de
   jr    loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_INIT
;;      Establishes everything with 0 entities 
;;      so that you can begin constructing new ones.
;; DESTROYS: AF, HL, DE, BC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_init::
   ;;----------------------------------------------------
   ;; Initialize _entity_num
   ;;
   xor   a                       ;; a = 0
   ld    (#_entity_num), a
    
   ;;----------------------------------------------------
   ;; Set all positions of m_entities in memory as 0s
   ;;
   ld    de, #_entity_array
   xor   a                        ;; a = 0
   ld    bc, #SIZE_ALL_ENTITIES
   call  cpct_memset_asm

   ;;----------------------------------------------------
   ;; Increment Array end pointer to point to the next
   ;; free element in the array
   ;;
   ld    hl, #_entity_array
   ld    (#_entity_pend), hl

   ;;----------------------------------------------------
   ;; Initialize _entity_zero_type_at_the_end = e_type_invalid
   ;;
   ld    a, #e_type_invalid
   ld    (#_entity_zero_type_at_the_end), a

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_NEW
;;    Adds new entity to the array without initializing it.
;; DESTROYS: BC, DE, HL
;; RETURN: 
;;    DE: Points to added entity
;;    BC: Sizeof Entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_new::
   ;;----------------------------------------------------
   ;; Increment number of reserved entities
   ;;
   ld    hl, #_entity_num
   inc   (hl)                        ;; m_num_entities++

   ;;----------------------------------------------------
   ;; Increment Array end pointer to point to the next
   ;; free element in the array
   ;;
   ld    hl, (#_entity_pend)
   ld    d, h                        ;;/ DE = HL
   ld    e, l                        ;;\ DE points to added entity
   ld    bc, #sizeof_e
   add   hl, bc
   ld    (#_entity_pend), hl

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_DELETEALL
;;    Erase all entities
;; DESTROYS: AF, DE, IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_deleteAll::
   call  man_entity_getArray     ;; IX = Entity Array 

next_entity_for_destruction:   
   ;;----------------------------------------------------
   ;; Check the end of the array and if it is, 
   ;; then returns
   ;;
   ld    a, (ix)        ;; a = entity type
   or    a
   ret   z

   call  man_entity_set4destruction       ;; Receives IX entity for destruction

   ld    de, #sizeof_e
   add   ix, de
   jr    next_entity_for_destruction

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_SET4DESTRUCTION
;;    Change entity type to e_type_dead
;; INPUT:
;;    IX: Pointer to entity to be destroyed
;; DESTROYS: AF, IX, HL, DE, BC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_destroy::
   push  hl       ;; HL = applied function
   ;;----------------------------------------------------
   ;; Check if it is the last created entity.
   ;; If (hl) = invalid entity, is the last
   ;;
   push  ix
   pop   hl
   ld    bc, #sizeof_e
   add   hl, bc
   ld    a, (hl)           ;; entity type
   cp    #e_type_invalid
   jr    z, conditional_entity_destroy 

   ;;----------------------------------------------------
   ;; Copy a bytes from last entity of the array 
   ;; in memory to destroyed one
   ;;
   ;;----------------------------------------------------
   ;; IX --> DE, DE = Pointer to the destination
   ;;  
   push  ix
   pop   de

   ld    hl, (#_entity_pend)     ;; Pointer to the next free entity
   ld    bc, #neg_sizeof_e
   add   hl, bc                  ;; HL = Pointer to the source
   ld    bc, #sizeof_e

   call cpct_memcpy_asm
conditional_entity_destroy:
   ld    hl, (#_entity_pend)     ;; Pointer to the next free entity
   ld    bc, #neg_sizeof_e
   add   hl, bc                  ;; HL = Pointer to the source
   
   ld    bc, #sizeof_e
   ld    a, #0
   call  cpct_memset_asm
   
   ;;----------------------------------------------------
   ;; Decrease Array end pointer to point to the next
   ;; free element in the array
   ;;
   ld    hl, (#_entity_pend)
   ld    bc, #neg_sizeof_e

   add   hl, bc
   ld    (#_entity_pend), hl

   ;;----------------------------------------------------
   ;; Decrease number of created entities
   ;;  
   ld    hl, #_entity_num
   dec   (hl)

   ;;----------------------------------------------------
   ;; Recover stored way_back(de) into hl, then they will be
   ;; exchanged. DE = applied function, HL = way_back
   ;;
   pop   de
   pop   hl
   jp    (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_DESTROY
;;    Destroys the entity, taking the last valid one and relocating it
;;    into its position
;; INPUT:
;;    IX: Pointer to entity to be destroyed
;; DESTROYS: AF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_set4destruction::
   ld    a, (ix)
   or    #e_type_dead
   ld    (ix), a

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_UPDATE
;;    Update entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_update::
   ld    c, #e_type_dead
   ld    hl, #man_entity_destroy
   call  man_entity_forall_matching_types
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_CREATE
;;    Creates and initializes new entity.
;; INPUT:
;;    HL: Pointer to initializer values for the entity to be created
;; STACK USE: 2 bytes
;; RETURN:
;;    IX: Pointer to created entity 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_create::
   push  hl                         ;; Save pointer to the initialization
   call  man_entity_new             ;; Add a new entity

   ;;----------------------------------------------------
   ;; IX = DE (IX returns the value of the pointer to
   ;; initialize entity)
   ;;  
   ld__ixh_d            ;;/ IX = DE
   ld__ixl_e            ;;\ IX Points to the new entity space
    
   ;;----------------------------------------------------
   ;; Copy initialization values to new entity
   ;; DE points to the new added entity
   ;; BC holds sizeof(Entity)
   ;; Copy initialization values
   ;;  
   pop   hl
   ldir        

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_FORALL_MATCHING_TYPES
;;    Applies particular function on all entities that match the type.
;; INPUT:
;;     C: Type of entity
;;    HL: Pointer to function to be applied
;; STACK USE: 6 bytes
;; DESTROYS: A, BC, IX, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_forall_matching_types::
   ;;----------------------------------------------------
   ;; IX = _entity_array, initialize iterator
   ;; 
   ld    ix, #_entity_array
check_entity:
   ;;----------------------------------------------------
   ;; Check the end of the array and if it is, 
   ;; then returns
   ;;
   ld    a, (ix)        ;; a = entity type
   or    a
   ret   z

   ld    b, (ix)
   ld    a, c           ;; a = matching type
   and   b              ;; and on a with b

   ;;----------------------------------------------------
   ;; If a is the same as it was, the entity is matching
   ;; 
   cp    c
   jr    nz, next_entity_to_check   ;; entity is not matching

   push  bc                         ;; Save matching type
   ;;----------------------------------------------------
   ;; Before executing the function, ensure the way back
   ;;
   ld    de, #way_back
   push  de
   
   ;;----------------------------------------------------
   ;; IX contains an entity. Execute function.
   ;; Make sure that the given function doesn't destroy HL
   ;; 
   jp    (hl)
way_back:
   pop   bc          ;; Recover matching type
   ex    de, hl      ;; HL = applied function and DE = way back
next_entity_to_check:
   ld    de, #sizeof_e
   add   ix, de
   jr    check_entity   
