;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_entity.h.s"
.include "man/game.h.s"
.include "man/entity.h.s"
.include "man/maps.h.s"
.include "man/animations.h.s"

.include "sys/render.h.s"
.include "sys/physics.h.s"
.include "sys/input.h.s"
.include "sys/animations.h.s"
.include "sys/collisions.h.s"
.include "sys/ai_control.h.s"

.include "assets/assets.h.s"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Game_Manager
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_GAME_INIT
;;    Initializes the Game Manager to set it up for start of 
;;    a new game.
;; DESTROYS: AF, BC, DE, HL, IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_init::
   ;;----------------------------------------------------
   ;; Init Entity Manager
   ;;
   call  man_entity_init
   
   ;;----------------------------------------------------
   ;; Init Level 0
   ;;
   call  man_maps_init

   ;;----------------------------------------------------
   ;; Init Systems
   ;;
   call  sys_ai_control_init
   call  sys_collisions_init
   call  sys_physics_init
   call  sys_input_init
   call  sys_render_init
   call  sys_animations_init

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_GAME_UPDATE
;;    Updates 1 Game Cycle doing everything except the rendering.
;;    Rendering is considered apart for its time constraints.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_update::
   ;;----------------------------------------------------
   ;; Update Maps Manager
   ;;
   call  man_maps_update

   ;;----------------------------------------------------
   ;; Update Entity Manager
   ;;
   call  man_entity_update

   ;;----------------------------------------------------
   ;; Update Systems
   ;;
   call  sys_input_update
   call  sys_ai_control_update
   call  sys_physics_update
   call  sys_collisions_update
   call  sys_animations_update
    
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_GAME_RENDER
;;    Does rendering process apart from game Update to account
;;    for its time constraints.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_render::
   call  sys_render_update
ret
