;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_array.h.s"
.include "cmp/macros_entity.h.s"

.include "man/entity.h.s"
.include "man/maps.h.s"
.include "man/game.h.s"
.include "man/animations.h.s"

.include "sys/render.h.s"
.include "sys/physics.h.s"

.include "main.h.s"

.include "assets/assets.h.s"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Maps_Manager
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Define types for entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 player_type = e_type_physics_mask + e_type_render_mask + e_type_animation_mask
 enemy_type  = e_type_render_mask + e_type_animation_mask + e_type_collider_mask + e_type_ai_mask
 candy_type  = e_type_render_mask + e_type_animation_mask + e_type_collider_mask
 door_type   = e_type_render_mask + e_type_animation_mask + e_type_collider_mask

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Manager Member Variables
;; DefineCmp_Entity  _type, _collider, _x, _y, _vx, _vy, _h, _w, _sprite, _anim_ptr,_target_x, _target_y  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 player: DefineCmp_Entity player_type, e_collider_none, MAIN_CHAR_X, MAIN_CHAR_Y, 0, 0, SP_MAIN_CHARACTER_H, SP_MAIN_CHARACTER_W, _sp_main_character, _main_char_inactive, 0, 0
 enemy:  DefineCmp_Entity enemy_type, e_collider_enemy, ENEMY_INIT_X, ENEMY_Y, 0, 0, SP_PUMPKIN_ENEMY_H, SP_PUMPKIN_ENEMY_W, _sp_pumpkin_enemy_0, _enemy_moving, ENEMY_INIT_X, ENEMY_FIN_X
 candy:  DefineCmp_Entity candy_type, e_collider_candy, 0, 0, 0, 0, 12, 7, _sp_candy_0, _candy_moving, 0, 0
 door:   DefineCmp_Entity door_type, e_collider_door, 0, 0, 0, 0, 16, 7, _sp_door_close, _door_closed, 0, 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VARIABLES IN MEMORY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_maps_array:
   .dw _mapa00, _mapa02, _mapa03, _mapa04 
    
current_map::
   .dw _mapa00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VARIABLES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   _game_status::       .db 0    ;; Alive = 1 / Dead = 2 / Open door = 3
   _door_status::       .db 0    ;; Closed = 0 / Open = 1
   _check_door::        .db 0    ;; Check = 1 / Not check = 0
   _current_level::     .db 0    ;; Current level number

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  PUBLIC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;man_maps_



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_MAPS_INIT
;;    Init level 0
;; DESTROYS: HL, IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_maps_init::
   ;;----------------------------------------------------
   ;; Draw Current Map
   ;;
   call  sys_render_tilemap

   ;;----------------------------------------------------
   ;; Draw Controls Instructions
   ;;
   ;;----------------------------------------------------
   ;; Calculate Video Memory Address where
   ;; draw the string
   ;;
   ;ld    de, #CPCT_VMEM_START_ASM   ;; Start of video memory
   ;ld    b, #8                      ;; Y Coord = 100 (pixels = bytes)
   ;ld    c, #16                     ;; X Coord = 40 (bytes)
   ;call  cpct_getScreenPtr_asm 
   
   ;;----------------------------------------------------
   ;; Draw the sprite
   ;;
   ;ex    de, hl                     ;; DE = pvmem (where to draw the sprite)
   ;ld    hl, #_img_controls         ;; Address where sprite data starts
   ;ld    b, #50                     ;; sprite width (bytes)
   ;ld    c, #16                     ;; sprite height (pixels = bytes)
   ;call  cpct_drawSprite_asm

   ;;----------------------------------------------------
   ;; Init Entities
   ;;
   ld    hl, #player          ;; Init pos: down-left corner
   call  man_entity_create
   
   ld    ix, #candy
   ld    e_x(ix), #35            ;; Init pos X
   ld    e_y(ix), #152           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #door
   ld    e_x(ix), #67            ;; Init pos X
   ld    e_y(ix), #136           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_MAPS_UPDATE
;;    Update level parametrs and checks if level is completed
;; DESTROYS: AF, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_maps_update::

   ;ld    a, (#_game_status)
   ;cp    #1
   ;jr    nz, _reset_lvl

   ld    a, (#_game_status)
   cp    #3
   jp    z, openDoor
_continue_update1:  
   ld    a, (#_check_door)
   cp    #1
   jp    z, checkOpenDoor
_continue_update2:

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_MAPS_CHANGE_LEVEL
;;    Change current map
;; DESTROYS: AF, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_maps_change_level::
   ;;----------------------------------------------------
   ;; Increment current level number (next level)
   ;; 
   ld    hl, #_current_level
   inc   (hl)

   ;;----------------------------------------------------
   ;; Store next map address into current_map
   ;; 
   ld    hl, #_maps_array
   ld    a, #NEXT_MAP   ;; level number 
   add_hl_a 
   ld    a, (hl)
   inc   hl
   ld    h, (hl)
   ld    l, a
   
   ld    (#current_map), hl        ;; HL = new map

   ;;----------------------------------------------------
   ;; Set all entities 4 destruction
   ;;
   call  man_entity_deleteAll
   
   ;;----------------------------------------------------
   ;; Initialize new Entities
   ;; Checks current level  
   ;;
   call  man_maps_init_entities_lvl

   ;;----------------------------------------------------
   ;; Initialize the data again
   ;;
   ld hl, #_game_status
   ld (hl), #1

   ld hl, #_door_status
   ld (hl), #0

   ld hl, #_check_door
   ld (hl), #0

   ;;----------------------------------------------------
   ;; Draw Current Map
   ;;
   call  sys_render_tilemap

   jr    _continue_check_open_door

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_MAPS_CHANGE_LEVEL
;;    Change current map
;; DESTROYS: DE, HL
;; RETURN:
;;    DE: Current map address
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_maps_get_current_de::
   ld  hl, #current_map
   ld  e, (hl)
   inc hl
   ld  d, (hl)
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; OPENDOOR
;;    Change door status and dor animation
;; DESTROYS: HL, AF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
openDoor:
   ;;----------------------------------------------------
   ;; Set door status to open
   ;;
   ld    hl, #_door_status
   ld    (hl), #1             ;; 1 is open, 0 is closed

   ;;----------------------------------------------------
   ;; Set game status to alive
   ;; door is already opened
   ;;
   ld    hl, #_game_status
   ld    (hl), #1

   ;;----------------------------------------------------
   ;; Get door entity in IX
   ;;
   ld    a, #e_collider_door
   call  man_entity_getEntityIX

   ;;----------------------------------------------------
   ;; Change door's sprite to open_door
   ;;
   ld    hl, #_sp_door_open_0
   ld    e_sprite_l(ix), l
   ld    e_sprite_h(ix), h

   ;;----------------------------------------------------
   ;; Change door's animation to open_door
   ;;
   ld    hl, #_door_opened
   ld    e_anim_ptr_l(ix), l
   ld    e_anim_ptr_h(ix), h

   jr    _continue_update1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CHECKOPENDOOR
;;    Check if the door is open, if so then change level
;; DESTROYS: HL, AF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
checkOpenDoor:
   ;;----------------------------------------------------
   ;; Set check_door to NOT_CHECK
   ;;
   ld hl, #_check_door
   ld (hl), #0

   ;;----------------------------------------------------
   ;; If door is open then change the level
   ;;
   ld    a, (#_door_status)
   cp    #1                   ;; if it's open
   jr    z, man_maps_change_level       ;; then next lvl
_continue_check_open_door:
   jr    _continue_update2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; _RESET_LVL
;;    Reset current level
;; DESTROYS: HL, AF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_reset_lvl:
   ;;----------------------------------------------------
   ;; Set all entities 4 destruction
   ;;
   call man_entity_deleteAll

   ;;  Initialize new Entities
   ;;call newEntities
   ;;  Initialize the data again
   ld hl, #_game_status
   ld (hl), #1

   ld hl, #_door_status
   ld (hl), #1
   ld hl, #_check_door
   ld (hl), #0

   ;;  Initialize the new map
   call man_maps_get_current_de
   call man_maps_init

ret





man_maps_init_entities_lvl:
   ;;----------------------------------------------------
   ;; If level 0
   ;;
   ld    a, #_current_level
   cp    #0
   jp    z, _entities_level_0

   ;;----------------------------------------------------
   ;; If level 1
   ;;
   ld    a, #_current_level
   cp    #1
   jp    z, _entities_level_1


_entities_level_0:
   ;;----------------------------------------------------
   ;; Init Entities
   ;;
   ld    hl, #player          ;; Init pos: down-left corner
   call  man_entity_create
   
   ld    ix, #candy
   ld    e_x(ix), #35            ;; Init pos X
   ld    e_y(ix), #152           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #door
   ld    e_x(ix), #67            ;; Init pos X
   ld    e_y(ix), #136           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ret
   
_entities_level_1:
   ;;----------------------------------------------------
   ;; Init Entities
   ;;
   ld    hl, #player          ;; Init pos: down-left corner
   call  man_entity_create
   
   ld    ix, #candy
   ld    e_x(ix), #67            ;; Init pos X
   ld    e_y(ix), #130           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #door
   ld    e_x(ix), #67            ;; Init pos X
   ld    e_y(ix), #40           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ret
      
_entities_level_2:
   ;;----------------------------------------------------
   ;; Init Entities
   ;;
   ld    hl, #player          ;; Init pos: down-left corner
   call  man_entity_create
   
   ld    hl, #enemy
   call  man_entity_create

   ld    ix, #enemy
   ld    e_y(ix), #80         ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #candy
   ld    e_x(ix), #5          ;; Init pos X
   ld    e_y(ix), #138        ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #door
   ld    e_x(ix), #4            ;; Init pos X
   ld    e_y(ix), #32           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ret

_entities_level_3:
   ;;----------------------------------------------------
   ;; Init Entities
   ;;
   ld    hl, #player          ;; Init pos: down-left corner
   call  man_entity_create
   
   ld    ix, #enemy
   ld    e_y(ix), #20         ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create


   ld    ix, #enemy
   ld    e_y(ix), #80         ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #candy
   ld    e_x(ix), #13          ;; Init pos X
   ld    e_y(ix), #58        ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ld    ix, #door
   ld    e_x(ix), #67           ;; Init pos X
   ld    e_y(ix), #96           ;; Init pos Y
   push  ix
   pop   hl
   call  man_entity_create

   ret