;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_entity.h.s"

.include "man/entity.h.s"
.include "man/animations.h.s"

.include "sys/animations.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Animations_System
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATIONS_INIT
;;    Initialize Animation system
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animations_init::
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATIONS_UPDATE_ONE_ENTITY
;;    Updates an Animation entity
;; INPUT:
;;    IX: Pointer to Animation Entity, it is not the end of the array
;; DESTROYS: AF, BC, HL, DE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animations_update_one_entity:
   call  calculateAnimStatus

   ;;----------------------------------------------------
   ;; Check if animation status is the same as before
   ;; 
   ld    a, e_anim_st(ix)        ;; A = Current Entity Anim Status
   cp    b
   jr    z, _continueAnim
_newanimationStatus:
   ld    e_anim_st(ix), b        ;; Store new Anim Status
   ex    de, hl                  ;; DE = applied function
   call  man_animations_getAnimHL
   ld    e_anim_ptr_l(ix), l
   ld    e_anim_ptr_h(ix), h
   ex    de, hl
_continueAnim:	
   ;;----------------------------------------------------
   ;; Current step in animation
   ;; 
   ld    e, e_anim_ptr_l(ix)
   ld    d, e_anim_ptr_h(ix)

   ;;----------------------------------------------------
   ;; HL has an applied function. hl and de have to 
   ;; be exchanged so not to lose their values.
   ;;
   ex    de, hl                  ;; HL = current animation
_return2Anim:	
   ld    c, (hl)                 ;;/
   inc   hl                      ;;| BC = Current sprite
   ld    b, (hl)                 ;;\
   inc   hl                      ;; HL += 2 (Points to next animation step)

   ;;----------------------------------------------------
   ;; If Current Sprite (BC) is nullptr
   ;; then reset Anim
   ;;
   ld    a, c
   or    b
   jr    z, _resetAnim

   ;;----------------------------------------------------
   ;; Store next animation step into the entity
   ;;
   ld    e_anim_ptr_l(ix), l
   ld    e_anim_ptr_h(ix), h

   ;;----------------------------------------------------
   ;; Store next sprite pointer into the entity
   ;;
   ld    e_sprite_l(ix), c
   ld    e_sprite_h(ix), b

   ;;----------------------------------------------------
   ;; Recover stored way_back(de) into hl, then they will be
   ;; exchanged. DE = applied function
   ;;
   pop   hl          ;; HL = way back
   jp    (hl)						

_resetAnim:
   ;;----------------------------------------------------
   ;; HL = (*HL)
   ;; HL points to the first step in the animation
   ;;
   ld    a, (hl)
   inc   hl
   ld    h, (hl)
   ld    l, a
   jr    _return2Anim

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATIONS_UPDATE
;;    Updates all Animation entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animations_update::
   ld    c, #e_type_animation_mask
   ld    hl, #sys_animations_update_one_entity
   call  man_entity_forall_matching_types
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CALCULATEANIMSTATUS
;;    Calculates animation status
;; DESTROYS: AF, BC
;; INPUT:
;;    IX = Entity_t
;; RETURN:
;;    B = Anim Status
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculateAnimStatus:
   ld    b, #0          ;; B = Original status (0)

   ;;----------------------------------------------------
   ;; If the entity is player, then continue
   ;;
   ld    a, e_collider(ix)
   ld    c, #e_collider_none  ;; a = matching type
   cp    c                
   ret   nz

   ;;----------------------------------------------------
   ;; Check left-right orientation
   ;;
   ld    a, e_vx(ix)    ;; A = e_vx
   or    a
   jr    z, _vxzero     ;; moving up or down
_vxnozero: 
   ;;----------------------------------------------------
   ;; Moving right or left
   ;;
   jp    m, _vxnegative	;; moving left
_vxpositive:
   ;;----------------------------------------------------
   ;; Moving right
   ;; 0010
   ;;
   set   1, b           ;; Set byte 1 to 1
   jr    _vxzero
_vxnegative:
   ;;----------------------------------------------------
   ;; Moving left
   ;; 0001
   ;;
   set   0, b
_vxzero:
   ;;----------------------------------------------------
   ;; Check up-down orientation
   ;;
   ld    a, e_vy(ix)    ;; A = e_vy
   or    a
   jr    z, _vyzero     ;; not moving
_vynozero:
   ;;----------------------------------------------------
   ;; Moving up or down
   ;;
   jp    m, _vynegative	;; moving up
_vypositive:
   ;;----------------------------------------------------
   ;; Moving down
   ;; 1000
   ;;
   set   3, b
   jr    _vyzero
_vynegative:
   ;;----------------------------------------------------
   ;; Moving up
   ;; 0100
   ;;
   set   2, b
_vyzero:
   ret
