;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_entity.h.s"

.include "man/entity.h.s"
.include "man/game.h.s"

.include "sys/physics.h.s"
.include "sys/collisions.h.s"
.include "sys/input.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Physics_System
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_INIT
;;    Initializes the physics system
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_init::
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_UPDATE_ONE_ENTITY
;;    Updates a Physics entity
;; INPUT:
;;    IX: Pointer to Physics Entity, it is not the end of the array
;; DESTROYS: AF, BC, HL, DE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_update_one_entity:
   ;;---------------------------------------------------- 
   ;; Update x
   ;;
   ld    a, e_x(ix)
   add   e_vx(ix)                   ;; next position of the player
   ld    e_x(ix), a                 ;; store next position into current
   
   ;;---------------------------------------------------- 
   ;; Update y
   ;;
   ;;---------------------------------------------------- 
   call sys_physics_gravity

_end:
   ;;----------------------------------------------------
   ;; HL = applied function, we exchange hl and de, so
   ;; not to lose the value of hl later.
   ;; Recover stored way_back(de) into hl, then they will
   ;; be exchanged in way_back.
   ;;
   ex    de, hl      ;; DE = applied function
   pop   hl          ;; HL = way_back
   jp    (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_UPDATE
;;    Updates all Physics entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_update::
   ld    c, #e_type_physics_mask
   ld    hl, #sys_physics_update_one_entity
   call  man_entity_forall_matching_types
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_UNDO_COLLISION
;;    Back to the last position in case of border collision
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

sys_physics_undo_collision::
   call  sys_physics_undo_collision_axis_X
   call  sys_physics_undo_collision_axis_Y

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_UNDO_COLLISION__AXIS_X
;;    Back to the last position in case of border collision in axis X
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_undo_collision_axis_X::
   ;;---------------------------------------------------- 
   ;; Update x
   ;;
   ld    a, e_x(ix)
   sub   e_vx(ix)
   ld    e_x(ix), a

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_UNDO_COLLISION__AXIS_Y
;;    Back to the last position in case of border collision in axis Y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_undo_collision_axis_Y::
   ;;---------------------------------------------------- 
   ;; Update y
   ;;
   ld    a, e_y(ix)
   sub   e_vy(ix)
   ld    e_y(ix), a

   ret  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_GRAVITY
;;    Imitate gravity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_gravity::
   ;;---------------------------------------------------- 
   ;; Gravity
   ;;
   inc   e_vy(ix)
   ld    a, e_y(ix)
   add   e_vy(ix)
   ld    e_y(ix), a

   ;;----------------------------------------------------
   ;; if e_y < 176 ret
   ;; 176 is 200 - sprite_h - border_h
   ;; 256 is total height of the screen
   ;;                    176              
   ;; 0 |-------y---------|-------| 256
   ;;
   add   #(256 - 176)
   ret   nc

   ;;----------------------------------------------------
   ;; if e_y > 176
   ;; e_vy = -20 if Q was pressed, then it is jump
   ;;
   ld    e_y(ix), #176     ;; sprite on the floor
   ld    e_vy(ix), #0
   
   ld    hl, #playerStatus
   ld    (hl), #1

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS_NO_GRAVITY
;;    Reset vy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_no_gravity::
   ld    e_vy(ix), #0

   ret
