;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/macros_entity.h.s"

.include "man/entity.h.s"
.include "man/maps.h.s"

.include "sys/render.h.s"

.include "assets/assets.h.s"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Render_System
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_RENDER_INIT
;;    Initializes the render system
;; DESTROYS: AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_init::
   ;;----------------------------
   ;; Initialize Amstrad CPC
   ;;
   ld    c, #0                      ;;/ Set Mode 0  
   call  cpct_setVideoMode_asm      ;;\ 160x200x16
   ld    hl, #_g_palette            ;;/
   ld    de, #16                    ;;| Set Palette
   call  cpct_setPalette_asm        ;;\
   cpctm_setBorder_asm HW_BLUE      ;; Border = Blue

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_RENDER_UPDATE_ONE_ENTITY
;;    Updates a Render entity
;; INPUT:
;;    IX: Pointer to Render Entity, it is not the end of the array
;; STACK USE: 2 bytes
;; DESTROYS: AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_update_one_entity:
   push  hl                       ;; Save applied function in stack

   ;;----------------------------------------------------
   ;; Erase Previous Instance
   ;;
   ld    d, e_lastVP_h(ix)
   ld    e, e_lastVP_l(ix)
   xor   a
   ld    b, e_h(ix)                ;; sprite width (bytes)
   ld    c, e_w(ix)                ;; sprite height (pixels = bytes)
   call  cpct_drawSolidBox_asm

   ;;----------------------------------------------------
   ;; Calculate Video Memory Address where
   ;; to draw the sprite
   ;;
   ld    de, #CPCT_VMEM_START_ASM   ;; Start of video memory
   ld    b, e_y(ix)                 ;; Y Coord = 100 (pixels = bytes)
   ld    c, e_x(ix)                 ;; X Coord = 40 (bytes)
   call  cpct_getScreenPtr_asm   
   ;; HL = pvmem

   ;;----------------------------------------------------
   ;; Store Video Memory Pointer at last position
   ;;
   ld    e_lastVP_l(ix), l
   ld    e_lastVP_h(ix), h

   ;;----------------------------------------------------
   ;; Draw the sprite
   ;;
   ex    de, hl                     ;; DE = pvmem (where to draw the sprite)
   ld    h, e_sprite_h(ix)          ;; Address where sprite data starts
   ld    l, e_sprite_l(ix)
   ld    b, e_h(ix)                 ;; sprite width (bytes)
   ld    c, e_w(ix)                 ;; sprite height (pixels = bytes)
   call  cpct_drawSprite_asm

   ;;----------------------------------------------------
   ;; Entity is set for destruction
   ;;
   ld    b, (ix)
   ld    a, #e_type_dead   ;; a = matching type
   and   b                 ;; and on a with b

   ;;----------------------------------------------------
   ;; If a is the same as it was, the entity is matching
   ;; 
   cp    #e_type_dead
   jr    nz, _end     ;; entity is not matching

   ;;----------------------------------------------------
   ;; Calculate Video Memory Address where
   ;; the sprite is
   ;;
   ld    de, #CPCT_VMEM_START_ASM   ;; Start of video memory
   ld    b, e_y(ix)                 ;; Y Coord = 100 (pixels = bytes)
   ld    c, e_x(ix)                 ;; X Coord = 40 (bytes)
   call  cpct_getScreenPtr_asm  
   ;;----------------------------------------------------
   ;; Erase Instance
   ;;
   ld    d, h
   ld    e, l
   xor   a
   ld    b, e_h(ix)                ;; sprite width (bytes)
   ld    c, e_w(ix)                ;; sprite height (pixels = bytes)
   call  cpct_drawSolidBox_asm

_end:
   ;;----------------------------------------------------
   ;; Recover stored values. We pop first the value that
   ;; was stored in hl(applied function) into de, so 
   ;; we can load the de value(way_back) into hl and
   ;; jump to this instruction. Later hl and de will be 
   ;; exchanged(in forall function).
   ;;
   pop   de          ;; DE = applied function
   pop   hl          ;; HL = way_back
   jp    (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_RENDER_UPDATE
;;    Updates all Render entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_update::
   ld    c, #e_type_render_mask
   ld    hl, #sys_render_update_one_entity
   call  man_entity_forall_matching_types
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_RENDER_TILEMAP
;;    Draws a map.
;; DESTROYS: AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_tilemap::
   ld    b, #25                  ;; Height in tiles
   ld    c, #20                  ;; Width in tiles
   ld    de, #20                 ;; Width
   ld    hl, #_mapatiles_00
   call  cpct_etm_setDrawTilemap4x8_ag_asm
   
   call  man_maps_get_current_de 
   ld    hl, #CPCT_VMEM_START_ASM
   call  cpct_etm_drawTilemap4x8_ag_asm
    
   ret
